package com.csc.library.zeesecurity.datasource.cassandra.manager;

import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.function.Function;

import javax.ejb.Singleton;
import javax.inject.Inject;

import com.csc.library.cassandra.Cassandra3Manager;
import com.csc.library.cassandra.MaterializedViewBuilder.OrderClustering;
import com.csc.library.cassandra.ViewContext;
import com.csc.library.cassandra.annotation.Column;
import com.csc.library.cassandra.util.PatternSqlBase;
import com.csc.library.entity.FileStore;
import com.csc.library.entity.manager.FileStoreUtilityManager;
import com.csc.library.status.StatusMessage;
import com.csc.library.system.layer.DatasourceEnvironment;
import com.csc.library.system.layer.DatasourceLayer;
import com.csc.library.system.layer.DatasourceType;
import com.csc.library.utilities.CSCCheckUtility;
import com.csc.library.utilities.CscCalendar;
import com.csc.library.utilities.CscCalendarSystem;
import com.csc.library.utilities.MyLog;
import com.csc.library.zeesecurity.databean.entity.CaptureTimestamp;
import com.csc.library.zeesecurity.databean.entity.impl.CaptureTimestampAbstract;
import com.csc.library.zeesecurity.databean.manager.CaptureTimestampManager;
import com.datastax.driver.core.ResultSet;
import com.datastax.driver.core.Row;
import com.datastax.driver.mapping.annotations.Accessor;
import com.datastax.driver.mapping.annotations.Query;

@Singleton
@DatasourceLayer(type=DatasourceType.CASSANDRA , env=DatasourceEnvironment.PRODUCTION)
public class CaptureTimestampCassandraManager extends Cassandra3Manager
	implements CaptureTimestampManager{

	@Inject
	private FileStoreUtilityManager fileManager;
	
	private Accessors accessors;
	private Function<Row , Optional<CaptureTimestamp>> fnConvertToEntity = r -> convertRowToEntity(r);

	@Override
	protected Class<?> getInterfaceDefineColumns() {
		return F.class;
	}

	@Override
	protected String[] getPartitionKey() {
		return new String[] { F.COMPANYID };
	}

	@Override
	protected String[] getClusteringKey() {
		return new String[] { F.CAPTUREID };
	}

	@Override
	protected void constructor() {
		this.accessors = this.createAccessors(Accessors.class);
	}

	@Override
	protected String getTableName() {
		return Accessors.TABLE_NAME;
	}
	
	@Override
	protected void defineMaterializedView(ViewContext context) {
		context.newViewInstance()
			.setViewName(Accessors.VW_SWIPEDATETIME)
			.setPrimaryKey(F.COMPANYID , F.SWIPEDATETIME , F.CAPTUREID)
			.setClusteringOrder(OrderClustering.DESC.wrap(F.SWIPEDATETIME))
			.register();
	}
	
	private interface F extends PatternSqlBase{
		@Column
		public final String COMPANYID = "COMPANYID";
		@Column
		public final String CAPTUREID = "CAPTUREID";
		@Column
		public final String FULLNAME = "FULLNAME";
		@Column(dataType="timestamp")
		public final String SWIPEDATETIME = "SWIPEDATETIME";
		@Column
		public final String LATITUDE = "LATITUDE";
		@Column
		public final String LOGITUDE = "LOGITUDE";
		@Column
		public final String REASON = "REASON";
		@Column(dataType="int")
		public final String SWIPETYPE = "SWIPETYPE";
		@Column
		public final String FILENAME = "FILENAME";
		
		public final String PATTERN_COLUMN = "(" 
				+ COMPANYID + "," + CAPTUREID  + "," + FULLNAME + ","+ SWIPEDATETIME + "," + LATITUDE
				+ "," + LOGITUDE + "," + REASON + "," + SWIPETYPE + "," + FILENAME
				+ ")";
		public final String PATTERN_COLUMN_VALUE = " VALUES("+ "?,?,?,?,?"
				+ ",?,?,?,?)";
	}
	
	@Accessor
	private interface Accessors extends F{
		public final String TABLE_NAME = "CAPTURE_TIMESTAMP";
		public final String VW_SWIPEDATETIME = TABLE_NAME + "_" + SWIPEDATETIME;
		
		public final String QUERY_TABLE_WHERE = PATTERN_SELECT + TABLE_NAME + PATTERN_WHERE;
		public final String QUERY_VW_SWIPEDATETIME_WHERE = PATTERN_SELECT + VW_SWIPEDATETIME + PATTERN_WHERE;
		
		
		@Query(QUERY_TABLE_WHERE + COMPANYID + " = ?")
		public ResultSet searchByCompanyId(String comId);
		
		@Query(QUERY_VW_SWIPEDATETIME_WHERE + COMPANYID + " = ? AND " + SWIPEDATETIME + " >= ? AND " + SWIPEDATETIME + " <= ?")
		public ResultSet searchByCompanyIdBetweenDatetime(String comId , Date start , Date end);
		
		@Query(PATTERN_INSERT + TABLE_NAME + PATTERN_COLUMN + PATTERN_COLUMN_VALUE + PATTERN_NOT_EXISTS)
		public ResultSet insert(String companyId,String captureId , String fullname , Date swipeDatetime , String latitude
				, String logitude , String reason , int swipeType , String fileName);
		
		@Query(PATTERN_INSERT + TABLE_NAME + PATTERN_COLUMN + PATTERN_COLUMN_VALUE)
		public ResultSet save(String companyId,String captureId , String fullname , Date swipeDatetime , String latitude
				, String logitude , String reason , int swipeType , String fileName);
	}
	
	private class CaptureTimestampEntity extends CaptureTimestampAbstract{
		private String captureId;
		private String companyId;
		
		private CaptureTimestampEntity(){}

		public String getCaptureId() {	return captureId;	}
		public void setCaptureId(String captureId) {	this.captureId = captureId;	}
		public String getCompanyId() {	return companyId;	}
		public void setCompanyId(String companyId) {	this.companyId = companyId;	}
	}
	
	private CaptureTimestampEntity createEntity(){
		return new CaptureTimestampEntity();
	}
	
	private Optional<CaptureTimestamp> convertRowToEntity(Row r){
		try{
			CaptureTimestampEntity e = this.createEntity();
			e.setCompanyId(r.getString(F.COMPANYID));
			e.setCaptureId(r.getString(F.CAPTUREID));
			e.setFullName(r.getString(F.FULLNAME));
			e.setSwipeType(r.getInt(F.SWIPETYPE));
			e.setLatitude(r.getString(F.LATITUDE));
			e.setLogitude(r.getString(F.LOGITUDE));
			e.setReason(r.getString(F.REASON));
			e.setFilename(r.getString(F.FILENAME));
			e.setSwipeDatetime(CscCalendarSystem.getFullDatetime(r.getTimestamp(F.SWIPEDATETIME)));
			return Optional.of(e);
		}catch(Exception e){
			MyLog.error(this , e);
		}
		return Optional.empty();
	}
	
	public CaptureTimestamp createInstance(String companyId){
		CaptureTimestampEntity e = this.createEntity();
		e.setCompanyId(companyId);
		e.setCaptureId(this.genUuid());
		return e;
	}
	
	public StatusMessage save(CaptureTimestamp ct){
		try{
			String filename = this.adjustFiletimestamp(ct);
			this.accessors.save(ct.getCompanyId(), ct.getCaptureId()
					, ct.getFullName(), CscCalendarSystem.parse(ct.getSwipeDatetime())
					, ct.getLatitude(), ct.getLogitude()
					, ct.getReason(), ct.getSwipeType()
					, ct.getFilename());
			return StatusMessage.createInstanceSuccess();
		}catch(Exception e){
			MyLog.error(this , e);
			return StatusMessage.createInstanceFail(e);
		}
	}
	
	private String adjustFiletimestamp(CaptureTimestamp ct){
		if(!CSCCheckUtility.isBlankOrNull(ct.getFilename()))
			return ct.getFilename();
		if(CSCCheckUtility.chkNullBytesToBytes(ct.getImage()).length == 0)
			return BLANK;
		try{
			FileStore fs = this.fileManager.createInstance(ct.getImage());
			this.fileManager.saveWithAdjustOldFileAndTrigger(fs, ct.getFilename() , f ->{
				ct.setFilename(f.getFileUuid());
			});
			return ct.getFilename();
		}catch(Exception e){
			MyLog.error(this , e);
		}
		return BLANK;
	}
	
	public List<CaptureTimestamp> findCaptureTimestampByCompany(String companyId){
		try{
			if(CSCCheckUtility.isBlankOrNull(companyId)) return Collections.emptyList();
			
			return fetchResultSet(this.accessors.searchByCompanyId(companyId), this.fnConvertToEntity);
		}catch(Exception e){
			MyLog.error(this , e);
		}
		return Collections.emptyList();
	}

	public List<CaptureTimestamp> findCaptureTimestampByCompanyAndBetweenDatetime(String companyId , CscCalendar start ,CscCalendar end){
		try{
			if(CSCCheckUtility.isBlankOrNull(companyId) && CSCCheckUtility.isNull(start , end)) return Collections.emptyList();
			
			return fetchResultSet(this.accessors.searchByCompanyIdBetweenDatetime(companyId
					, CscCalendarSystem.parse(start.getTimeInMillis()), CscCalendarSystem.parse(end.getTimeInMillis())), this.fnConvertToEntity);
		}catch(Exception e){
			MyLog.error(this , e);
		}
		return Collections.emptyList();
	}
}
