package com.csc.library.zeesecurity.datasource.cassandra.manager;

import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.function.Function;

import javax.ejb.Singleton;

import com.csc.library.cassandra.Cassandra3Manager;
import com.csc.library.cassandra.annotation.Column;
import com.csc.library.cassandra.util.PatternSqlBase;
import com.csc.library.status.StatusMessage;
import com.csc.library.system.layer.DatasourceEnvironment;
import com.csc.library.system.layer.DatasourceLayer;
import com.csc.library.system.layer.DatasourceType;
import com.csc.library.utilities.CSCCheckUtility;
import com.csc.library.utilities.CscCalendarSystem;
import com.csc.library.utilities.MyLog;
import com.csc.library.zeesecurity.databean.entity.EmployeeReplacement;
import com.csc.library.zeesecurity.databean.entity.impl.EmployeeReplacementAbstract;
import com.csc.library.zeesecurity.databean.manager.EmployeeReplacementManager;
import com.datastax.driver.core.ResultSet;
import com.datastax.driver.core.Row;
import com.datastax.driver.mapping.annotations.Accessor;
import com.datastax.driver.mapping.annotations.Query;


@Singleton
@DatasourceLayer(type=DatasourceType.CASSANDRA , env=DatasourceEnvironment.PRODUCTION)
public class EmployeeReplacementCassandraManager
	extends Cassandra3Manager
	implements EmployeeReplacementManager{
	
	private Accessors accessors;
	private Function<Row , Optional<EmployeeReplacement>> fnConvertToEntity = r -> convertRowToEntity(r);

	@Override
	protected Class<?> getInterfaceDefineColumns() {
		return F.class;
	}

	@Override
	protected String[] getPartitionKey() {
		return new String[]{ F.COMPANYID };
	}

	@Override
	protected String[] getClusteringKey() {
		return new String[] { F.MEMBERID , F.REPLACEMENTID };
	}

	@Override
	protected void constructor() {
		this.accessors = this.createAccessors(Accessors.class);
	}

	@Override
	protected String getTableName() {
		return Accessors.TABLE_NAME;
	}

	private interface F extends PatternSqlBase{
		@Column
		public final String COMPANYID = "COMPANYID";
		@Column
		public final String MEMBERID = "MEMBERID";
		@Column
		public final String REPLACEMENTID = "REPLACEMENTID";
		@Column
		public final String MEMBERID_REPLACEMENT = "MEMBERID_REPLACEMENT";
		@Column(dataType="timestamp")
		public final String STARTDATE = "STARTDATE";
		@Column(dataType="timestamp")
		public final String ENDDATE = "ENDDATE";
		
		public final String PATTERN_COLUMN = "(" 
				+ COMPANYID + "," + MEMBERID  + "," + REPLACEMENTID + ","+ MEMBERID_REPLACEMENT + "," + STARTDATE
				+ "," + ENDDATE
				+ ")";
		public final String PATTERN_COLUMN_VALUE = " VALUES("+ "?,?,?,?,?"
				+ ",?)";
	}
	
	@Accessor
	private interface Accessors extends F{
		public final String TABLE_NAME = "EMPLOYEE_REPLACEMENT";
		public final String QUERY_TABLE_WHERE = PATTERN_SELECT + TABLE_NAME + PATTERN_WHERE;
		
		@Query(QUERY_TABLE_WHERE + COMPANYID + " = ?")
		public ResultSet searchByCompanyId(String comId);
		
		@Query(PATTERN_INSERT + TABLE_NAME + PATTERN_COLUMN + PATTERN_COLUMN_VALUE + PATTERN_NOT_EXISTS)
		public ResultSet insert(String companyId,String memberId , String repId , String memberIdRep , Date startDate
				, Date endDate);
		
		@Query(PATTERN_INSERT + TABLE_NAME + PATTERN_COLUMN + PATTERN_COLUMN_VALUE)
		public ResultSet save(String companyId,String memberId , String repId , String memberIdRep , Date startDate
				, Date endDate);
	}
	
	private class EmployeeReplacementEntity extends EmployeeReplacementAbstract{
		private String companyId;
		private String memberId;
		private String replacementId;
		private EmployeeReplacementEntity(){}
		public String getCompanyId() {	return companyId; }
		public void setCompanyId(String companyId) {	this.companyId = companyId;	}
		public String getMemberId() {	return memberId;	}
		public void setMemberId(String memberId) {	this.memberId = memberId;	}
		public String getReplacementId() {	return replacementId;	}
		public void setReplacementId(String replacementId) {	this.replacementId = replacementId;	}
	}
	
	private EmployeeReplacementEntity createEntity(){
		return new EmployeeReplacementEntity();
	}
	
	private Optional<EmployeeReplacement> convertRowToEntity(Row r){
		try{
			EmployeeReplacementEntity e = this.createEntity();
			e.setCompanyId(r.getString(F.COMPANYID));
			e.setMemberId(r.getString(F.MEMBERID));
			e.setReplacementId(r.getString(F.REPLACEMENTID));
			e.setMemberIdReplacement(r.getString(F.MEMBERID_REPLACEMENT));
			e.setStartDate(this.convertDatetimeToString(r.getTimestamp(F.STARTDATE)));
			e.setEndDate(this.convertDatetimeToString(r.getTimestamp(F.ENDDATE)));
			return Optional.of(e);
		}catch(Exception e){
			MyLog.error(this , e);
		}
		return Optional.empty();
	}
	
	private String convertDatetimeToString(Date date){
		try{
			return CscCalendarSystem.getFullDatetime(date);
		}catch(Exception e){
			return BLANK;
		}
	}
	private Date convertStringToDatetime(String datetime){
		try{
			return CscCalendarSystem.parse(datetime);
		}catch(Exception e){
			return null;
		}
	}
	public EmployeeReplacement createInstance(String companyId, String memberId){
		EmployeeReplacementEntity e = this.createEntity();
		e.setCompanyId(companyId);
		e.setMemberId(memberId);
		e.setReplacementId(this.genUuid());
		return e;
	}
	public List<EmployeeReplacement> findEmployeeReplacementByCompany(String companyId){
		try{
			if(CSCCheckUtility.isBlankOrNull(companyId)) return Collections.emptyList();
			
			return fetchResultSet(this.accessors.searchByCompanyId(companyId), this.fnConvertToEntity);
		}catch(Exception e){
			MyLog.error(this , e);
		}
		return Collections.emptyList();
	}
	
	public StatusMessage save(EmployeeReplacement model){
		try{
			if(CSCCheckUtility.isNotNull(model)){
				this.accessors.save(model.getCompanyId(), model.getMemberId()
						, model.getReplacementId(), model.getReplacementId()
						, this.convertStringToDatetime(model.getStartDate()), this.convertStringToDatetime(model.getEndDate()));
				return StatusMessage.createInstanceSuccess();
			}else{
				return StatusMessage.createInstanceFail();
			}
		}catch(Exception e){
			MyLog.error(this , e);
			return StatusMessage.createInstanceFail(e);
		}
	}
}
